/*
	appscheduler.cpp - applications scheduler
	Copyright (C) 2003  Konrad Twardowski <kdtonline@poczta.onet.pl>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "appscheduler.h"
#include "kshutdowndcopapi.h"
#include "msystemtray.h"

#include <dcopclient.h>
#include <kdebug.h>
#include <klocale.h>

/* constructor: AppInfo */

AppInfo::AppInfo(const int id, const QString &name, const QString &description, const int action)
	: QObject()
{
	_action = action;
	_id = id;
	_appId = name;
	_description = description;
	_name = name;
}

/* destructor: AppInfo */

AppInfo::~AppInfo()
{
}

/* constructor: AppScheduler */

AppScheduler::AppScheduler()
	:
	// Iface
	DCOPObject("KShutdownIface"),
	QValueList<AppInfo *>(),
	// init
	_enabled(TRUE)
{
	newId = mainWindow->winId();
}

/* destructor: AppScheduler */

AppScheduler::~AppScheduler()
{
}

/* public: AppScheduler */

void AppScheduler::removeDead()
{
// TODO: AppScheduler::removeDead()
#if 0
	if (!isEnabled())
		return;

	QValueList<AppInfo *>::iterator it = begin();
	DCOPClient *client = kapp->dcopClient();
	QCString cs;
	for (; it != end(); ++it)
	{
		cs = (*it)->getAppId();
		if ((*it)->getId() && !client->isApplicationRegistered(cs))
		{
			kdDebug() << "AppScheduler::removeDead(): removing dead task: " << cs << endl;
			(*it)->setId(0);
			(*it)->setDescription((*it)->getDescription() + " (" + i18n("Error") + ")");
		}
	}
#endif
}

MMainWindow::What AppScheduler::translateAction(const int action) const
{
	switch (action)
	{

		case ACTION_SHUT_DOWN:
			return MMainWindow::What_ShutDown;

		case ACTION_REBOOT:
			return MMainWindow::What_Reboot;

		case ACTION_LOGOUT:
			return MMainWindow::What_Logout;

		default:
			return MMainWindow::What_Nothing;

	}
}

// DCOP

void AppScheduler::cancel()
{
	mainWindow->cancelAction();
}

bool AppScheduler::shutDown()
{
	return mainWindow->execAction(MMainWindow::What_ShutDown);
}

bool AppScheduler::shutdown()
{
	return shutDown();
	//.........^^
}

bool AppScheduler::reboot()
{
	return mainWindow->execAction(MMainWindow::What_Reboot);
}

bool AppScheduler::lockScreen()
{
	return mainWindow->execAction(MMainWindow::What_LockScreen);
}

bool AppScheduler::logout()
{
	return mainWindow->execAction(MMainWindow::What_Logout);
}

bool AppScheduler::activateAction(int id)
{
	kdDebug() << "DCOP: AppScheduler::activateAction: id = `" << id << "'" << endl;

	if (!isEnabled())
	{
		showStatusInfo();

		return FALSE;
	}

	QValueList<AppInfo *>::iterator it = findId(id); // find task by id
	if (it == 0) // task not found
	{
		kdDebug() << "DCOP: AppScheduler::activateAction: `" << id << "' is not registered!" << endl;

		// inform user
		QString text = i18n("The task is not registered!");
		text += " (id = " + QString::number(id) + ")";
		mainWindow->tray()->popupMessage(MSystemTray::Reason_Error, mainWindow->caption(), text, 20);

		return FALSE; // error
	}

// TODO: priority
	MMainWindow::What actionToExecute = translateAction((*it)->getAction());

	if (actionToExecute == MMainWindow::What_Nothing)
		return FALSE; // error

	QString desc = (*it)->getDescription(); // save before unregister!
	unregisterTask(id); // unregister task, deref.

	// ok, but wait for other registered tasks
	if (count() != 0)
		return TRUE; // ok

	// no more tasks, so we can shut down, reboot or logout
	mainWindow->setCancelled(FALSE);

	// no warning message, immediate action execute
	if (!mainWindow->warningMessageEnabled())
		return mainWindow->execAction(actionToExecute, FALSE);

	// warning message
	// - create non-modal dialog
	// - after specified time, action will be continued
	// - user can cancel the action
	mainWindow->messageDialogAccepted(FALSE, actionToExecute, desc);

	return TRUE; // ok
}

int AppScheduler::registerTask(const QString &name, const QString &description, int action)
{
	kdDebug() << "DCOP: AppScheduler::registerTask: action = `" << action << "'" << endl;

	if (!isEnabled())
	{
		showStatusInfo();

		return 0;
	}

	if (
		(action != ACTION_SHUT_DOWN) &&
		(action != ACTION_REBOOT) &&
		(action != ACTION_LOGOUT)
	)
	{
// TODO: show a nice error message

		return 0;

	}

	newId++;
	QString desc = description + " (" + mainWindow->getActionName(translateAction(action)) + ")";
	append(new AppInfo(
		newId,
		name,
		desc,
		action)
	);

	// inform user
	mainWindow->tray()->popupMessage(
		MSystemTray::Reason_Info,
		mainWindow->caption(),
		name + ": " + desc,
		20
	);

	// update info
	mainWindow->updateSchedulerTab(TRUE);

	return newId;
}

bool AppScheduler::unregisterTask(int id)
{
	if (!isEnabled())
	{
		showStatusInfo();

		return FALSE;
	}

	// find task
	QValueList<AppInfo *>::iterator it = findId(id);
	if (it != 0)
	{
		remove(it);
		// update info
		mainWindow->updateSchedulerTab();

		return TRUE;
	}

	return FALSE;
}

QString AppScheduler::getStatusInfo()
{
	return mainWindow->caption();
}

void AppScheduler::setTestMode(bool yes)
{
	mainWindow->setTestMode(yes);
}

void AppScheduler::wizard()
{
	mainWindow->runWizard();
}

/* private: AppScheduler */

QValueList<AppInfo *>::iterator AppScheduler::findId(int id)
{
	QValueList<AppInfo *>::iterator it = begin();
	for (; it != end(); ++it)
	{

		if ((*it)->getId() == id)
			return it; // found

	}

	return 0; // not found
}

void AppScheduler::showStatusInfo()
{
	// inform user
	mainWindow->tray()->popupMessage(
		MSystemTray::Reason_Error,
		mainWindow->caption(),
		i18n("The scheduler is disabled!"),
		20
	);
}
