/*
	mmainwindow.h - main window
	Copyright (C) 2003  Konrad Twardowski <kdtonline@poczta.onet.pl>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __MMAINWINDOW_H__
#define __MMAINWINDOW_H__

#include <qcheckbox.h>
#include <qdatetimeedit.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qtabwidget.h>

#include <kapplication.h>
#include <kdatewidget.h>
#include <kmainwindow.h>
#include <kpushbutton.h>

#define mainWindow MMainWindow::mainAppWindow()

class AppScheduler;
class Extras;
class MMessageDialog;
class MSchedulerTab;
class MStatsTab;
class MSystemTray;

/** @short Main window. */
class MMainWindow: public KMainWindow
{
	Q_OBJECT
public:
	/** Delay types (When_Delay). Don't change these values! */
	enum DelayType {
		/** Delay in seconds. */
		DelayType_Seconds = 0,
		/** Delay in minutes. */
		DelayType_Minutes = 1,
		/** Delay in hours. */
		DelayType_Hours = 2
	};

	/** Action methods. Don't change these values! */
	enum Method {
		/** Standard KDE/KDM method (default). */
		Method_KDE = 0,
		/** Default external command (e.g. /sbin/reboot). */
		Method_DefaultCommand = 1,
		/**
		 * User command.
		 * Any command that can be executed by @c KRun::run.
		 */
		Method_UserCommand = 2
	};

	/** Action types. */
	enum What {
		/** Nothing (null). */
		What_Nothing = 0,
		/** System shut down. */
		What_ShutDown = 1,
		/** System reboot. */
		What_Reboot = 2,
		/** Screen lock. */
		What_LockScreen = 3,
		/** End current session. */
		What_Logout = 4,
		/** Extras. Execute an item selected from the "Extras..." menu. */
		What_Extras = 5
	};

	/** Timeout options. Don't change these values! */
	enum When {
		/** Immediate action (no delay). */
		When_Now = 0,
		/**
		 * Delay in seconds, minutes, or hours.
		 * @see DelayType
		 */
		When_Delay = 1,
		/** Time from now in HH:MM format. */
		When_DelayTime = 2,
		/** Absolute date and time. */
		When_DateTime = 3
	};

	/**
	 * Constructor.
	 * @param parent Parent widget
	 * @param name Widget name
	 */
	MMainWindow(QWidget *parent = 0, const char *name = 0);

	/** Destructor. */
	virtual ~MMainWindow();

	/**
	 * Returns @c TRUE if the action has been cancelled by the user.
	 * Used in @ref MMessageDialog.
	 */
	inline bool actionCancelled() const { return _cancelled; }

	/** Returns the "Application scheduler". */
	inline AppScheduler *appScheduler() const { return _appScheduler; }

	/** Cancels current action. (--cancel) */
	void cancelAction();

	/** Returns the main configuration file. */
	inline KConfig *config() const { return kapp->config(); };

	/** Returns the active action. */
	inline What currentAction() const { return what; }

	/**
	 * Ends current session. This function is a wrapper for the @c kapp->requestShutDown().
	 * @return @c TRUE if successful; otherwise @c FALSE
	 */
	bool endSession(const KApplication::ShutdownType type, const What action);

	/**
	 * Executes an action.
	 * @param action Action to execute
	 * @param stopTimer If @c TRUE, the active action and timer are stopped
	 * @return @c TRUE if successful; otherwise @c FALSE
	 */
	bool execAction(const What action, const bool stopTimer = TRUE);

	/**
	 * Returns an action icon.
	 * @param action Action
	 */
	QPixmap getActionIcon(const What action) const;

	/**
	 * Returns an action name as text.
	 * @param action Action
	 * @see getCurrentActionName
	 */
	QString getActionName(const What action) const;

	/**
	 * Returns current action name as text.
	 * @see getActionName
	 */
	inline QString getCurrentActionName() const { return getActionName(what); }

	/**
	 * Reads method for action from the configuration file.
	 * @param configEntry Configuration entry
	 * @param defaultCommand Default command
	 * @retval method Method to use
	 * @retval command Command to run
	 */
	void getMethod(
		const QString &configEntry,
		Method &method,
		QString &command,
		const QString &defaultCommand);

	/** Returns @c TRUE if action is active. */
	inline bool isActive() const { return _active; }

	/**
	 * Returns @c TRUE if @p key is restricted (KDE Kiosk).
	 * The key value is read from the configuration file
	 * (usually /usr/share/config/kshutdownrc). See README for details.
	 */
	inline bool isRestricted(const QString &key) const { return !kapp->authorize(key); };

	/** Returns the main window widget. Accessed via @c mainWindow macro. */
	inline static MMainWindow *mainAppWindow() { return mainApplicationWindow; }

	/**
	 * Creates and shows a new modal or non-modal message dialog.
	 * Non-modal message dialog is used in @ref AppScheduler.
	 * @param modal If @c TRUE, the dialog is modal and blocks the main window.
	 * @param actionToExecute Action to execute after timeout, or after button click
	 * @param text Text to display
	 */
	bool messageDialogAccepted(
		const bool modal,
		const What actionToExecute,
		const QString &text);

	/**
	 * Visual and sound notifications.
	 * @param secs Seconds to the shut down, reboot, etc.
	 */
	void notifyUser(const int secs) const;

	/**
	 * Plays a sound file. The extension will be automatically added.
	 * @param dir Sound directory
	 * @param name Sound name
	 */
	void play(const QString &dir, const QString &name) const;

	/**
	 * Runs the Wizard.
	 * @return @c TRUE if successfull; otherwise @c FALSE
	 */
	bool runWizard();

	/**
	 * Activates/deactivates an action.
	 * @param action Action to activate/deactivate
	 * @param yes @c TRUE - activate, @c FALSE - deactivate
	 * @see isActive
	 */
	void setActive(const What action, const bool yes);

	/**
	 * Notifies one or more message dialog that all actions have been cancelled.
	 * @see actionCancelled
	 */
	inline void setCancelled(const bool yes) { _cancelled = yes; }

	/** Sets editor date to @p date. */
	inline void setDate(const QDate &date) { de_date->setDate(date); }

	/**
	 * Sets delay type to @p value.
	 * NOTE: This is not the @ref setWhen!
	 */
	void setDelayType(const DelayType value);

	/** Sets spin box value to @p value. */
	inline void setDelayValue(const int value) { sb_delay->setValue(value); }

	/**
	 * Enables/disables "test mode".
	 * @param yes If @c TRUE, the test mode is enabled
	 */
	inline void setTestMode(const bool yes)
	{
		_testMode = yes;
		setWindowCaption(""); // add/remove "[ TEST MODE ]"
	}

	/** Sets editor time to @c time. */
	inline void setTime(const QTime &time)
	{
		te_time->setTime(QTime(time.hour(), time.minute(), 0));
	}

	/**
	 * Sets delay type to @p value.
	 * @see When
	 */
	void setWhen(const When value);

	/**
	 * Displays "KRun" error message.
	 * @param command Command name that failed
	 */
	void showRunErrorMessage(const QString &command);

	/** Returns @c TRUE if test mode is enabled. (--test option) */
	inline bool testMode() const { return _testMode; }

	/** Returns the system tray. */
	inline MSystemTray *tray() const { return systemTray; }

	/**
	 * Updates the "Scheduler" tab.
	 * @param focus If @c TRUE the tab is raised/focused
	 * @see MSchedulerTab::update
	 */
	void updateSchedulerTab(const bool focus = FALSE);

	/** Returns @c TRUE if the warning message is enabled. */
	inline bool warningMessageEnabled() const { return _warningMessageEnabled; }
private:
	AppScheduler
		*_appScheduler;
	bool
		_active,
		_cancelled,
		_testMode,
		totalExit;
	DelayType
		delayType;
	Extras
		/** "Extras" manager. */
		*extras;
	int
		margin,
		spacing,
		oldSec;
	KDateWidget
		/** Date editor. */
		*de_date;
	KPushButton
		// main buttons
		*b_shutDown,
		*b_reboot,
		*b_lockScreen,
		*b_logout,
		*b_wizard,
		*b_extras,
		*b_cancel,
		// time tab
		*b_misc;
	/** Points to the main window widget. */
	static MMainWindow *mainApplicationWindow;
	MMessageDialog
		*_modalMessageDialog;
	MSchedulerTab
		*schedulerTab;
	MStatsTab
		*statsTab;
	MSystemTray
		*systemTray;
	QCheckBox
		/** "Remember above settings" check box. */
		*c_rememberTime;
	QComboBox
		*cb_delayType,
		*cb_when;
	QDateTime
		/** End date/time. */
		*dt_end,
		/** Start date/time. */
		*dt_start;
	QLabel
		*l_date,
		*l_timeFromNow,
		*l_timeHHMM;
	QSpinBox
		/** Delay editor (seconds, minutes, or hours). */
		*sb_delay;
	QTabWidget
		/** Tabs. */
		*tabWidget;
	QTimeEdit
		/** Time editor. */
		*te_time;
	QTimer
		/** Timeout timer. */
		*timer;
	QWidget
		*timeTab;
	What
		/** Selected action. */
		what;
	When
		/**
		 * Selected timeout option.
		 * @see cb_when
		 */
		when;
	// configuration
	bool
		confirmAction,
		soundEnabled,
		_warningMessageEnabled;
	int
		warningMessageDelay;
	QString
		/** Sound directory. */
		soundDir;

	/**
	 * Displays confirmation message box to confirm selected action.
	 * @param action Action to confirm
	 * @return @c TRUE if action has been confirmed; otherwise @c FALSE
	 */
	bool confirm(const What action);

	QWidget *initBottomButtons(QWidget *parent);
	QWidget *initButtons(QWidget *parent);
	QWidget *initTimeTab(QWidget *parent);

	/**
	 * Plays the sound.
	 * @param name Sound name
	 * @code
	 * playSound("one");
	 * @endcode
	 * @see play
	 */
	inline void playSound(const QString &name) const
	{
		if (soundEnabled)
			play(soundDir, name);
	}

	/** Reads the configuration. Called automatically at application startup. */
	void readConfig();

	/**
	 * Executes the command.
	 * @param command Command to execute
	 * @return @c TRUE if successfull; otherwise @c FALSE
	 */
	bool runCommand(const QString &command);

	/**
	 * Converts seconds to time string.
	 * @code
	 * QString s = secsToTimeStr(70); // s = "00:01:10"
	 * @endcode
	 * @param secs Seconds to convert
	 * @return Converted seconds or ">24", if @p secs >= 86400
	 */
	QString secsToTimeStr(const int secs) const;

	/** Sets window and system tray caption to @p text. */
	void setWindowCaption(const QString &text);

	/**
	 * Displays the settings dialog.
	 * @param pageName Page name to show
	 */
	void showSettingsDialog(const QString &pageName = QString::null);

	/** Executes the active action after timeout. */
	void totalShutDown();

	/**
	 * Updates window caption.
	 * Displays time to the final shut down, and the selected action name.
	 */
	void updateWindowCaption(const int secs);

	/** Writes the configuration. Called automatically at application exit. */
	void writeConfig();
private slots:
	/**
	 * This signal is emitted when the application is about to quit, or when the
	 * user shuts down the entire desktop session.
	 */
	void aboutToQuitSlot();

	/** Checks the time, and after timeout executes active action. */
	void checkTimeSlot();

	/**
	 * Called when a new item has been selected in the "Delay type" combo box.
	 * @param index New item index
	 */
	void delayTypeChangeSlot(int index);

	/** Resets date/time in the date or time editor. */
	void miscSlot();

	/** Displays the settings dialog. */
	void settingsSlot();

	/**
	 * Called when a new tab has been selected.
	 * Used to initialize tabs "on demand".
	 */
	void tabChangedSlot(QWidget *tab);

	/**
	 * Called when a new item has been selected in the "When?" combo box.
	 * @param index New item index
	 */
	void whenChangeSlot(int index);

	/** Launches the Wizard. */
	void wizardSlot();
public slots:
	/** Cancels an active action. */
	void cancelSlot();
	/** Exists the application. */
	void closeSlot();
	/** Locks the screen. */
	void lockScreenSlot();
	/** Ends current session. */
	void logoutSlot();
	/** Reboots the machine. */
	void rebootSlot();
	/** Halts the machine. */
	void shutDownSlot();
protected:
	/* Closes or minizes the main window. */
	virtual bool queryClose();
	/**
	 * Invoked at application exit.
	 * @return @c TRUE
	 */
	virtual bool queryExit();
};

#endif // __MMAINWINDOW_H__
