/*
	msettingsdialog.cpp - settings dialog
	Copyright (C) 2003  Konrad Twardowski <kdtonline@poczta.onet.pl>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mactioneditdialog.h"
#include "mmessagedialog.h"
#include "msettingsdialog.h"
#include "msystemtray.h"
#include "mtip.h"

#include <qlayout.h>
#include <qtimer.h>
#include <qvgroupbox.h>
#include <qwhatsthis.h>

#include <kconfig.h>
#include <kiconloader.h>
#include <klineedit.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <krun.h>
#include <kstandarddirs.h>

/* constructor */

MSettingsDialog::MSettingsDialog(QWidget *parent, const char *name)
	: KDialogBase(
		IconList,
		i18n("Settings"),
// TODO: help
		Ok | Cancel | Default,
		Ok, // default button
		parent,
		name,
		TRUE, // modal
		TRUE	// separator
		),
	// init
	soundTest(0)
{
	initGeneralPage();
	initSoundPage();
	initActionsPage();
	initSchedulerPage();
}

/* destructor */

MSettingsDialog::~MSettingsDialog()
{
}

/* public */

QString MSettingsDialog::getDefaultSoundDir()
{
	QStringList dirs;
	dirs = KGlobal::dirs()->findDirs("data", "kshutdown/sounds");
	QStringList::ConstIterator it = dirs.begin();
	if (it != dirs.end())
		// pick first
		return *it;

	// return something
	return "/usr/share/apps/kshutdown/sounds/";
}

void MSettingsDialog::setWarningMessage(const bool on, const int delay)
{
	warningMessageDelay->setValue(delay);
	warningMessageEnabled->setChecked(on);
	toggleWarningMessageSlot(on);
}

void MSettingsDialog::showPageByName(const QString &name)
{
	if (name.isNull())
		return;

	if (name == "General")
		showPage(pageIndex(generalPage));
	else if (name == "Sound")
		showPage(pageIndex(soundPage));
	else if (name == "Actions")
		showPage(pageIndex(actionsPage));
	else if (name == "Scheduler")
		showPage(pageIndex(schedulerPage));
}

/* private */

bool MSettingsDialog::disabledByAdmin(const QString &key, QFrame *page, QBoxLayout *pageLayout) const
{
	if (mainWindow->isRestricted("settings_" + key))
	{
		// add tip
		if (page)
		{
			MTip *adminTip = new MTip(MTip::TipType_Info, page, "MTip::adminTip");
			adminTip->setTipText(i18n("This page has been disabled by the Administator."));
			pageLayout->addWidget(adminTip);
		}

		return TRUE; // disabled
	}

	return FALSE; // enabled
}

void MSettingsDialog::initActionsPage()
{
	actionsPage = addPage(i18n("Actions"), QString::null, MainBarIcon(SETTINGS_ICON_ACTIONS));
	QVBoxLayout *actionsLayout = new QVBoxLayout(actionsPage, 5, 5);

	if (disabledByAdmin("actions", actionsPage, actionsLayout))
		return;

	actions = new QListView(actionsPage, "QListView::actions");
	actions->setAllColumnsShowFocus(TRUE);
	actions->setItemMargin(5);
	actions->setSorting(-1); // no sort
	actions->addColumn(i18n("Action"));
	actions->addColumn(i18n("Method / Command"));

	logoutItem = new QListViewItem(actions, mainWindow->getActionName(MMainWindow::What_Logout));
	logoutItem->setPixmap(0, mainWindow->getActionIcon(MMainWindow::What_Logout));

	lockScreenItem = new QListViewItem(actions, mainWindow->getActionName(MMainWindow::What_LockScreen));
	lockScreenItem->setPixmap(0, mainWindow->getActionIcon(MMainWindow::What_LockScreen));

	rebootItem = new QListViewItem(actions, mainWindow->getActionName(MMainWindow::What_Reboot));
	rebootItem->setPixmap(0, mainWindow->getActionIcon(MMainWindow::What_Reboot));

	shutDownItem = new QListViewItem(actions, mainWindow->getActionName(MMainWindow::What_ShutDown));
	shutDownItem->setPixmap(0, mainWindow->getActionIcon(MMainWindow::What_ShutDown));

	connect(
		actions, SIGNAL(doubleClicked(QListViewItem *, const QPoint &, int)),
		this, SLOT(editActionSlot()));
	connect(
		actions, SIGNAL(spacePressed(QListViewItem *)),
		this, SLOT(editActionSlot()));

	actionsLayout->addWidget(actions);

	// edit action
	KPushButton *b_edit = new KPushButton(i18n("&Edit..."), actionsPage, "KPushButton::b_edit");
	connect(
		b_edit, SIGNAL(clicked()),
		this, SLOT(editActionSlot()));
	actionsLayout->addWidget(b_edit);

// TODO: modify "extras" from here

	// tip
	MTip *actionsTip = new MTip(MTip::TipType_Normal, actionsPage, "MTip::actionsTip");
	actionsTip->setTipText(
		"<ul>" \
		"<li>" + i18n("If you are using <b>KDE</b> and <b>KDM</b> (KDE Display Manager), then set all methods to <i>KDE</i>") + "</li>" \
		"<li>" + i18n("If you are using <b>KDE</b> and display manager different than <b>KDM</b>, then set <i>Shut Down</i> and <i>Reboot</i> methods to <i>/sbin/...</i>") + "</li>" \
		"<li>" + i18n("If you are running <b>KShutDown</b> from the non-KDE session (e.g. <b>GNOME</b>), then change all methods...") + "</li>" \
		"<li>" + i18n("See FAQ for more details") + "</li>" \
		"</ul>"
		);
	actionsLayout->addWidget(actionsTip);

	// update items after widgets init
	updateActions();
}

void MSettingsDialog::initGeneralPage()
{
	generalPage = addPage(i18n("General"), QString::null, MainBarIcon(SETTINGS_ICON_GENERAL));
	QVBoxLayout *generalLayout = new QVBoxLayout(generalPage, 5, 5);

	// confirm action
	confirmAction = new QCheckBox(i18n("Co&nfirm Action"), generalPage, "QCheckBox::confirmAction");
	generalLayout->addWidget(confirmAction);

	// popup messages
	QWidget *popupMessagesWidget = new QWidget(generalPage, "QWidget::popupMessagesWidget");
	QHBoxLayout *popupMessagesLayout = new QHBoxLayout(popupMessagesWidget);
	popupMessages = new QCheckBox(i18n("&Popup Messages (Passive)"), popupMessagesWidget, "QCheckBox::popupMessages");
	popupMessagesLayout->addWidget(popupMessages);
	KPushButton *popupMessagesTest = new KPushButton(i18n("Test"), popupMessagesWidget, "KPushButton::popupMessagesTest");
	connect(
		popupMessagesTest, SIGNAL(clicked()),
		this, SLOT(popupMessagesTestSlot()));
	popupMessagesLayout->addWidget(popupMessagesTest);
	generalLayout->addWidget(popupMessagesWidget);

	// warning message:
	QVGroupBox *gb_warning = new QVGroupBox(i18n("&Warning Message"), generalPage, "QVGroupBox::gb_warning");

	// enabled/info
	warningMessageEnabled = new QCheckBox("", gb_warning, "QCheckBox::warningMessageEnabled");
	connect(
		warningMessageEnabled, SIGNAL(toggled(bool)),
		this, SLOT(toggleWarningMessageSlot(bool)));

	// slider
	warningMessageDelay = new KIntNumInput(gb_warning, "KIntNumInput::warningMessageDelay");
	warningMessageDelay->setSuffix(" " + i18n("minute(s)"));
	warningMessageDelay->setRange(1, 60, 1, TRUE);
	warningMessageDelay->setValue(DEFAULT_WARNING_MESSAGE_DELAY);
	connect(
		warningMessageDelay, SIGNAL(valueChanged(int)),
		this, SLOT(updateWarningMessageSlot()));
	KPushButton *warningMessageTest = new KPushButton(i18n("Test"), gb_warning, "KPushButton::warningMessageTest");
	connect(
		warningMessageTest, SIGNAL(clicked()),
		this, SLOT(warningMessageTestSlot()));
	generalLayout->addWidget(gb_warning);

	// lock screen settings
	b_lockScreenSettings = new KPushButton(mainWindow->getActionIcon(MMainWindow::What_LockScreen), i18n("Scree&n Saver Settings..."), generalPage, "KPushButton::b_lockScreenSettings");
	connect(
		b_lockScreenSettings, SIGNAL(clicked()),
		this, SLOT(lockSettingsSlot()));
	generalLayout->addWidget(b_lockScreenSettings);

	// logout settings
	b_logoutSettings = new KPushButton(mainWindow->getActionIcon(MMainWindow::What_Logout), i18n("Logo&ut Settings..."), generalPage, "KPushButton::b_logoutSettings");
	connect(
		b_logoutSettings, SIGNAL(clicked()),
		this, SLOT(logoutSettingsSlot()));
	generalLayout->addWidget(b_logoutSettings);

	// -
	generalLayout->addStretch();

	// warning
	MTip *t_warning = new MTip(MTip::TipType_Warning, generalPage, "MTip::t_warning");
	t_warning->setTipText(i18n("In most cases you need privileges to shut down system (e.g. run /sbin/reboot or /sbin/shutdown)") + "<br><br>" + i18n("See also: <i>Actions</i>."));
	generalLayout->addWidget(t_warning);

	updateWarningMessageSlot();
}

void MSettingsDialog::initSchedulerPage()
{
	schedulerPage = addPage(i18n("Scheduler"), QString::null, MainBarIcon(SETTINGS_ICON_SCHEDULER));
	QVBoxLayout *schedLayout = new QVBoxLayout(schedulerPage, 5, 5);

	// confirm action
	schedulerEnabled = new QCheckBox(i18n("&Enabled"), schedulerPage, "QCheckBox::schedulerEnabled");
	schedLayout->addWidget(schedulerEnabled);

	// -
	schedLayout->addStretch();
}

void MSettingsDialog::initSoundPage()
{
	soundPage = addPage(i18n("Sound"), QString::null, MainBarIcon(SETTINGS_ICON_SOUND));
	QVBoxLayout *soundLayout = new QVBoxLayout(soundPage, 5, 5);

	// sound enabled
	soundEnabled = new QCheckBox(i18n("&Enabled"), soundPage, "QCheckBox::soundEnabled");
	soundLayout->addWidget(soundEnabled);

	// sound dir label
	QLabel *l_soundDir = new QLabel(i18n("So&und Directory"), soundPage);
	soundLayout->addWidget(l_soundDir);

	// sound dir requester
	soundDir = new KURLRequester(soundPage, "KURLRequester::soundDir");
	l_soundDir->setBuddy(soundDir);
	soundDir->setMode(KFile::Directory | KFile::ExistingOnly);
	QWhatsThis::add(soundDir, "<qt>" + i18n("Select a sound directory.<br><br>Press <b>Defaults</b> button to restore the default sound directory.") + "</qt>");
	soundLayout->addWidget(soundDir);

	// test sound button
	KPushButton *b_testSound = new KPushButton(i18n("Test"), soundPage, "KPushButton::b_testSound");
	soundLayout->addWidget(b_testSound);
	connect(
		b_testSound, SIGNAL(clicked()),
		this, SLOT(testSoundSlot()));

	// -
	soundLayout->addStretch();
}

void MSettingsDialog::setMethod(const QString &configEntry, const MMainWindow::Method method, const QString &command) const
{
	KConfig *config = mainWindow->config();
	config->setGroup(configEntry);
	if (method == MMainWindow::Method_UserCommand)
		config->writeEntry("Command", command);
	config->writeEntry("Method", method);
}

void MSettingsDialog::updateActions()
{
	updateItem(shutDownItem);
	updateItem(rebootItem);
	updateItem(lockScreenItem);
	updateItem(logoutItem);
	actions->setCurrentItem(shutDownItem);
}

void MSettingsDialog::updateItem(QListViewItem *item)
{
	if (!item)
		return;

	MMainWindow::Method m = MMainWindow::Method_KDE;
	QString c;

	if (item == shutDownItem)
	{
		mainWindow->getMethod("Shut Down", m, c, DEFAULT_SHUT_DOWN_COMMAND);
	}
	else if (item == rebootItem)
	{
		mainWindow->getMethod("Reboot", m, c, DEFAULT_REBOOT_COMMAND);
	}
	else if (item == lockScreenItem)
	{
		mainWindow->getMethod("Lock Screen", m, c, DEFAULT_LOCK_SCREEN_COMMAND);
	}
	else if (item == logoutItem)
	{
		mainWindow->getMethod("Logout", m, c, DEFAULT_LOGOUT_COMMAND);
	}

	// method name
	if (m == MMainWindow::Method_KDE)
		item->setText(1, "KDE");
	else
		item->setText(1, c);
}

/* private slots */

void MSettingsDialog::editActionSlot()
{
	QListViewItem *item = actions->currentItem();

	if (!item)
		return;

	MMainWindow::Method m = MMainWindow::Method_KDE;
	QString c;

	MActionEditDialog *dialog = new MActionEditDialog(this, "MActionEditDialog::dialog");

	Q_CHECK_PTR(dialog);

	dialog->methods->insertItem("KDE");

	if (item == shutDownItem)
	{
		mainWindow->getMethod("Shut Down", m, c, DEFAULT_SHUT_DOWN_COMMAND);
		dialog->methods->insertItem(DEFAULT_SHUT_DOWN_COMMAND);
		dialog->setWhat(MMainWindow::What_ShutDown);
	}
	else if (item == rebootItem)
	{
		mainWindow->getMethod("Reboot", m, c, DEFAULT_REBOOT_COMMAND);
		dialog->methods->insertItem(DEFAULT_REBOOT_COMMAND);
		dialog->setWhat(MMainWindow::What_Reboot);
	}
	else if (item == lockScreenItem)
	{
		mainWindow->getMethod("Lock Screen", m, c, DEFAULT_LOCK_SCREEN_COMMAND);
		dialog->methods->insertItem(DEFAULT_LOCK_SCREEN_COMMAND);
		dialog->setWhat(MMainWindow::What_LockScreen);
	}
	else if (item == logoutItem)
	{
		mainWindow->getMethod("Logout", m, c, DEFAULT_LOGOUT_COMMAND);
		dialog->methods->insertItem(DEFAULT_LOGOUT_COMMAND);
		dialog->setWhat(MMainWindow::What_Logout);
	}
	dialog->methods->insertItem(i18n("User Command"));

	dialog->setUserCommand(c);
	dialog->setMethod(m);

	if (dialog->exec() == QDialog::Accepted)
	{
		m = (MMainWindow::Method)dialog->methods->currentItem();
		if (m == MMainWindow::Method_UserCommand)
			c = dialog->command->text();
		// kdDebug() << "MSettingsDialog::editActionSlot(): m = `" << m << "'" << endl;
		// kdDebug() << "MSettingsDialog::editActionSlot(): c = `" << c << "'" << endl;

		if (item == shutDownItem)
		{
			setMethod("Shut Down", m, c);
		}
		else if (item == rebootItem)
		{
			setMethod("Reboot", m, c);
		}
		else if (item == lockScreenItem)
		{
			setMethod("Lock Screen", m, c);
		}
		else if (item == logoutItem)
		{
			setMethod("Logout", m, c);
		}

		updateItem(item);
	}

	delete dialog;
}

void MSettingsDialog::enableLockSettingsButtonSlot()
{
	b_lockScreenSettings->setEnabled(TRUE);
}

void MSettingsDialog::enableLogoutSettingsButtonSlot()
{
	b_logoutSettings->setEnabled(TRUE);
}

void MSettingsDialog::lockSettingsSlot()
{
	b_lockScreenSettings->setEnabled(FALSE);
	QTimer::singleShot(5000, this, SLOT(enableLockSettingsButtonSlot()));
	KRun::run("kcmshell screensaver", KURL::List());
}

void MSettingsDialog::logoutSettingsSlot()
{
	b_logoutSettings->setEnabled(FALSE);
	QTimer::singleShot(5000, this, SLOT(enableLogoutSettingsButtonSlot()));
	KRun::run("kcmshell kcmsmserver", KURL::List());
}

void MSettingsDialog::popupMessagesTestSlot()
{
	mainWindow->tray()->popupMessage(
		MSystemTray::Reason_Info,
		i18n("Test"),
		i18n("Test"),
		10, // timeout
		TRUE // test
	);
}

void MSettingsDialog::testSoundSlot()
{
	QString s;

	s = soundDir->url();
	if (!s.endsWith("/"))
		s.append("/");

	QFileInfo fi(s + "one.ogg");
	if (fi.exists())
	{
		soundTest == 5 ? soundTest = 1 : soundTest++; // calc sound number
		switch (soundTest)
		{
			case 1:
				mainWindow->play(soundDir->url(), "one");
				break;
			case 2:
				mainWindow->play(soundDir->url(), "two");
				break;
			case 3:
				mainWindow->play(soundDir->url(), "one");
				break;
			case 4:
				mainWindow->play(soundDir->url(), "two");
				break;
			case 5:
				mainWindow->play(soundDir->url(), "three");
				break;
		}
	}
	else
	{
		s = QString("<qt>" + i18n("File <b>%1</b> not found.") + "</qt>")
			.arg(fi.filePath());
		KMessageBox::sorry(this, s);
	}
}

void MSettingsDialog::toggleWarningMessageSlot(bool on)
{
	warningMessageDelay->setEnabled(on);
}

void MSettingsDialog::updateWarningMessageSlot()
{
	QString s =
		QString(i18n("Show warning message %1 minute(s) &before shut down"))
		.arg(warningMessageDelay->value());
	warningMessageEnabled->setText(s);
}

void MSettingsDialog::warningMessageTestSlot()
{
	mainWindow->setCancelled(FALSE);
	MMessageDialog *messageDialog = new MMessageDialog(
		TRUE, warningMessageDelay->value(),
		MMainWindow::What_Nothing, i18n("Test"),
		this, "MMessageDialog::messageDialog"
	);
	messageDialog->exec();
	delete messageDialog;
}

/* protected slots */

void MSettingsDialog::slotDefault()
{
	int index = activePageIndex();

	if (index == -1)
		return;

	// general
	if (index == pageIndex(generalPage))
	{
		confirmAction->setChecked(TRUE);
		popupMessages->setChecked(TRUE);
		setWarningMessage(DEFAULT_WARNING_MESSAGE_ENABLED, DEFAULT_WARNING_MESSAGE_DELAY);
	}
	// sound
	else if (index == pageIndex(soundPage))
	{
		soundDir->setURL(getDefaultSoundDir());
		soundEnabled->setChecked(TRUE);
	}
	// actions
	else if (index == pageIndex(actionsPage))
	{
		if (disabledByAdmin("actions"))
			return;

		if (KMessageBox::warningYesNo(this, i18n("Restore default settings for this page?"), caption()) != KMessageBox::Yes)
			return;

		KConfig *config = mainWindow->config();
		config->deleteGroup("Shut Down");
		config->deleteGroup("Reboot");
		config->deleteGroup("Lock Screen");
		config->deleteGroup("Logout");
		updateActions();
	}
	// scheduler
	else if (index == pageIndex(schedulerPage))
	{
		schedulerEnabled->setChecked(TRUE);
	}
}
