/*
	mstatstab.cpp - statistics tab
	Copyright (C) 2003  Konrad Twardowski <kdtonline@poczta.onet.pl>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mmainwindow.h"
#include "mstatstab.h"

#include <qlayout.h>
#include <qregexp.h>
#include <qwhatsthis.h>

#include <kiconloader.h>
#include <klocale.h>

/* constructor */

MStatsTab::MStatsTab(QWidget *parent, const char *name)
	: QWidget(parent, name),
	// init
	initDone(FALSE),
	_longFormat(FALSE),
	b_refresh(0),
	c_longFormat(0),
	process(0),
	buf(""),
	te_output(0)
{
}

/* destructor */

MStatsTab::~MStatsTab()
{
	killProcess();
}

/* public */

void MStatsTab::initOnDemand()
{
	if (initDone)
	{
		refreshSlot();

		return;
	}

	process = new QProcess(this);

	connect(
		process, SIGNAL(processExited()),
		this, SLOT(processExitSlot()));
	connect(
		process, SIGNAL(readyReadStdout()),
		this, SLOT(readStdoutSlot()));

	QVBoxLayout *l = new QVBoxLayout(this, 4, 4);

	// output
	te_output = new QTextEdit(this, "QTextEdit::te_output");
	te_output->setPaletteBackgroundColor(QColor(0, 0, 0)); // black
	te_output->setPaletteForegroundColor(QColor(0, 255, 0)); // green
	te_output->setReadOnly(TRUE);
	te_output->setTextFormat(RichText); // HTML tags
	te_output->setWordWrap(QTextEdit::NoWrap);
	
	// based on the "Linux User's Manual" (man w)
	QWhatsThis::add(te_output,
		"<qt>" +
		i18n(
		"This view displays information about the users currently on the machine, and their processes.<br>" \
		"The header shows how long the system has been running."
		) +
		"</qt>"
	);
	
	l->addWidget(te_output);
	
	// long format option
	c_longFormat = new QCheckBox(i18n("Show logi&n time, JCPU and PCPU times"), this, "QCheckBox::c_longFormat");
	connect(
		c_longFormat, SIGNAL(toggled(bool)),
		this, SLOT(longFormatCheckedSlot(bool)));
	c_longFormat->setChecked(longFormat());
	
	l->addWidget(c_longFormat);

	// refresh button
	b_refresh = new KPushButton(SmallIcon("reload"), i18n("R&efresh"), this, "KPushButton::b_refresh");
	b_refresh->setFlat(TRUE);
	connect(
		b_refresh, SIGNAL(clicked()),
		this, SLOT(refreshSlot()));
	l->addWidget(b_refresh);
	
	// show created widgets
	te_output->show();
	c_longFormat->show();
	b_refresh->show();
	
	refreshSlot();
	
	initDone = TRUE;
}

void MStatsTab::setLongFormat(const bool yes)
{
	_longFormat = yes;
	if (initDone)
		c_longFormat->setChecked(_longFormat);
}

/* private */

void MStatsTab::killProcess() const
{
	// kill previous process if any
	if (process && process->isRunning())
		process->kill();
}

/* private slots */

void MStatsTab::longFormatCheckedSlot(bool check)
{
	setLongFormat(check);
	if (initDone)
		refreshSlot();
}

void MStatsTab::processExitSlot()
{
	buf.replace("\n", "<br>");
	buf.replace(" ", "&nbsp;");
	
	// underline header
	buf.replace("USER", "<font color=\"#ffffff\">USER</font>");
	buf.replace("TTY", "<font color=\"#ffffff\">TTY</font>");
	buf.replace("FROM", "<font color=\"#ffffff\">FROM</font>");
	buf.replace("LOGIN@", "<font color=\"#ffffff\">LOGIN@</font>");
	buf.replace("JCPU", "<font color=\"#ffffff\">JCPU</font>");
	buf.replace("PCPU", "<font color=\"#ffffff\">PCPU</font>");
	buf.replace("IDLE", "<font color=\"#ffffff\">IDLE</font>");
	buf.replace("WHAT", "<font color=\"#ffffff\">WHAT</font>");
	
	buf.replace(QRegExp("\\broot\\b"), "<font color=\"#ffff00\">root</font>");
	buf.replace(QRegExp("\\bssh\\b"), "<font color=\"#ff0000\">ssh</font>");
	buf.replace(QRegExp("\\bsu\\b"), "<font color=\"#ff0000\">su</font>");
	buf.replace(QRegExp("\\bsudo\\b"), "<font color=\"#ff0000\">sudo</font>");
	buf.replace(QRegExp("\\btelnet\\b"), "<font color=\"#ff0000\">telnet</font>");
	
	buf.prepend("<qt><code style=\"font-family: Luxi Mono, Courier\"><b>");
	buf.append("</b></code></qt>");

	te_output->setText(buf);

	c_longFormat->setEnabled(TRUE);
	b_refresh->setEnabled(TRUE);
}

void MStatsTab::readStdoutSlot()
{
	buf.append(process->readStdout());
}

void MStatsTab::refreshSlot()
{
	if (mainWindow->isRestricted("tab_stats"))
	{
		te_output->setText(
			"<font color=\"#ff0000\"><b>" +
			i18n("Disabled by the Administrator.") +
			"</b></font>"
		);
		setEnabled(FALSE);

		return;
	}

	killProcess();
	
	c_longFormat->setEnabled(FALSE);
	b_refresh->setEnabled(FALSE);
	
	buf = "";

	process->clearArguments();
	process->addArgument("w");
	if (!longFormat())
		process->addArgument("-s"); // -s - short format
	if (!process->start())
	{
		buf = "<font color=\"#ff0000\"><b>" + i18n("Error") + "</b></font>";
		buf.append("<br><br>" + i18n("Command:") + " ");
		buf.append(process->arguments().join(" "));
		te_output->setText(buf);
		c_longFormat->setEnabled(TRUE);
		b_refresh->setEnabled(TRUE);
	}
}
