/*
	msystemtray.cpp - system tray
	Copyright (C) 2003  Konrad Twardowski <kdtonline@poczta.onet.pl>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mmainwindow.h"
#include "msystemtray.h"

#include <qtimer.h>
#include <qtooltip.h>
#include <qvbox.h>

#include <kiconloader.h>
#include <klocale.h>
#include <kpixmapeffect.h>
#include <kpopupmenu.h>

/* constructor */

MSystemTray::MSystemTray(QWidget *parent, const char *name)
	: KSystemTray(parent, name),
	// init
	_idle(FALSE), // setIdle(TRUE) will toggle it
	_popupMessages(TRUE),
	flashCount(0),
	_popup(0), // initialized in popup()
	saveIcon(0)
{
	// init flash timer
	flashTimer = new QTimer(this);
	connect(
		flashTimer, SIGNAL(timeout()),
		this, SLOT(flashTimeoutSlot()));

	// init icons
	KPixmap pm(kapp->miniIcon());
	activeIcon = new KPixmap(KPixmapEffect::fade(pm, 0.7f, QColor(255, 255, 0)));

	flashIcon = new KPixmap(KPixmapEffect::fade(pm, 0.5f, QColor(255, 255, 255)));

	setIdle(TRUE);

	QToolTip::add(this, mainWindow->caption());
	show();

	// add some items to the context menu
	KPopupMenu *cm = contextMenu();

	KPopupMenu *group = new KPopupMenu(cm, "KPopupMenu::group");

	// shut down
	shutDownItem = group->insertItem(
		mainWindow->getActionIcon(MMainWindow::What_ShutDown), i18n("&Shut Down"), this, SLOT(shutDownSlot()));
	// reboot
	rebootItem = group->insertItem(
		mainWindow->getActionIcon(MMainWindow::What_Reboot), i18n("&Reboot"), this, SLOT(rebootSlot()));
	// -
	group->insertSeparator();
	// lock screen
	lockScreenItem = group->insertItem(
		mainWindow->getActionIcon(MMainWindow::What_LockScreen), i18n("Loc&k Screen"), this,
		SLOT(lockScreenSlot()));
	group->insertSeparator();
	// logout
	logoutItem = group->insertItem(
		mainWindow->getActionIcon(MMainWindow::What_Logout), i18n("&Logout"), this, SLOT(logoutSlot()));
	// confirm
// TODO:	confirmItem = group->insertItem(

	// 0 - title
	cm->insertItem(i18n("&Immediate Action"), group, -1, 1);
	// Wizard...
	wizardItem = cm->insertItem(
		SmallIcon("wizard"), i18n("&Wizard..."), this, SLOT(wizardSlot()), 0, -1, 2);
	// Cancel
	cancelItem = cm->insertItem(
		SmallIcon("button_cancel"), i18n("&Cancel"), mainWindow, SLOT(cancelSlot()), 0, -1, 3);
	// -
	cm->insertSeparator(4);

// TODO: help
	// Help
//	cm->insertItem(SmallIcon("help"), i18n("&Help"), mainWindow->helpMenu(), -1, 5);

	// Settings
	cm->insertItem(
		SmallIcon("configure"), i18n("C&onfigure..."), mainWindow, SLOT(settingsSlot()), 0, -1, 5);

	// quit signal
	connect(
		this, SIGNAL(quitSelected()),
		mainWindow, SLOT(closeSlot()));
}

/* destructor */

MSystemTray::~MSystemTray()
{
	delete activeIcon;
	delete flashIcon;
	if (_popup)
		delete _popup;
	if (saveIcon)
		delete saveIcon;
}

/* public */

void MSystemTray::flash()
{
	if (saveIcon)
		delete saveIcon;
	saveIcon = new QPixmap;
	*saveIcon = *pixmap();
	flashCount = 0;
	setPixmap(*flashIcon);
	flashTimer->start(200);
}

void MSystemTray::hidePopupMessages()
{
	if (_popup)
		_popup->hide();
}

KPassivePopup *MSystemTray::popup(const QString &caption)
{
	if (_popup == 0)
	{
		_popup = new KPassivePopup(this, "KPassivePopup::_popup");
		QVBox *popupView = new QVBox(_popup, "QVBox::popupView");
		// icon
		QLabel *popupIcon = new QLabel(popupView, "QLabel::popupIcon");
		popupIcon->setPixmap(kapp->miniIcon());
		// caption
		popupCaption = new QLabel(popupView, "QLabel::popupCaption");
		popupCaption->setText(caption);
		// messages list
		popupMessagesList = new QListBox(popupView, "QListBox::popupMessagesList");

		_popup->setView(popupView);
	}
	else
	{
		if (!caption.isNull())
			popupCaption->setText(caption);
	}

	return _popup;
}

void MSystemTray::popupMessage(const Reason reason, const QString &caption, const QString &text, const int timeout, const bool test)
{
	if (!popupMessages() && !test)
		return;

// TODO: timeout config
	popup(caption); // init popup
	popup()->setTimeout(timeout * 1000); // seconds
	QString t = QTime::currentTime().toString() + ": " + text;
	switch (reason)
	{
		case Reason_Info:
			popupMessagesList->insertItem(SmallIcon("messagebox_info"), t);
			break;
		case Reason_Warning:
			popupMessagesList->insertItem(SmallIcon("messagebox_warning"), t);
			break;
		case Reason_Error:
			popupMessagesList->insertItem(SmallIcon("messagebox_critical"), t);
			break;
		default:
			popupMessagesList->insertItem(t);
	}
	// focus last item
	popupMessagesList->setCurrentItem(popupMessagesList->count() - 1);

	popup()->show();
}

void MSystemTray::popupMessage(const Reason reason, const QString &text)
{
	popupMessage(reason, mainWindow->caption(), text);
}

void MSystemTray::setIdle(const bool yes)
{
	if (_idle == yes)
		return;

	_idle = yes;
	// idle
	if (_idle)
		setPixmap(kapp->miniIcon());
	// busy
	else
		setPixmap(*activeIcon);
}

/* protected */

void MSystemTray::contextMenuAboutToShow(KPopupMenu *menu)
{
	menu->setItemEnabled(shutDownItem, !mainWindow->isRestricted("action_shutdown"));
	menu->setItemEnabled(rebootItem, !mainWindow->isRestricted("action_reboot"));
	menu->setItemEnabled(lockScreenItem, !mainWindow->isRestricted("action_lockscreen"));
	menu->setItemEnabled(logoutItem, !mainWindow->isRestricted("action_logout"));
	menu->setItemEnabled(wizardItem, !mainWindow->isRestricted("action_wizard"));

	if (mainWindow->isActive())
	{
		menu->changeItem(cancelItem, i18n("&Cancel") + ": " + mainWindow->getCurrentActionName());
		menu->setItemEnabled(cancelItem, TRUE);
	}
	else
	{
		menu->changeItem(cancelItem, i18n("&Cancel"));
		menu->setItemEnabled(cancelItem, FALSE);
	}
}

/* private slots */

void MSystemTray::flashTimeoutSlot()
{
	flashCount++;

	if ((flashCount % 2) == 0)
		setPixmap(*saveIcon);
	else
		setPixmap(*flashIcon);

	if (flashCount == 6)
		flashTimer->stop();
}

void MSystemTray::lockScreenSlot()
{
	// don't stop timer
	mainWindow->execAction(MMainWindow::What_LockScreen, FALSE);
}

void MSystemTray::logoutSlot()
{
	mainWindow->execAction(MMainWindow::What_Logout);
}

void MSystemTray::rebootSlot()
{
	mainWindow->execAction(MMainWindow::What_Reboot);
}

void MSystemTray::shutDownSlot()
{
	mainWindow->execAction(MMainWindow::What_ShutDown);
}

void MSystemTray::wizardSlot()
{
	mainWindow->runWizard();
}
