/*
	msystemtray.h - system tray
	Copyright (C) 2003  Konrad Twardowski <kdtonline@poczta.onet.pl>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __MSYSTEMTRAY_H__
#define __MSYSTEMTRAY_H__

#include <qlistbox.h>

#include <kpassivepopup.h>
#include <kpixmap.h>
#include <ksystemtray.h>

/** @short System tray icon and popup messages. */
class MSystemTray: public KSystemTray
{
	Q_OBJECT
public:

	/** This value tells what kind of icon will be used in the list. */
	enum Reason {
		/** Information icon. */
		Reason_Info = 0,
		/** Warning icon. */
		Reason_Warning = 1,
		/** Error icon. */
		Reason_Error = 2
	};

	/**
	 * Constructor.
	 * @param parent Parent widget
	 * @param name Widget name
	 */
	MSystemTray(QWidget *parent, const char *name = 0);

	/** Destructor. */
	virtual ~MSystemTray();

	/** Flashes the system tray icon. */
	void flash();
	
	/** Hiddes the popup messages. */
	void hidePopupMessages();

	/**
	 * Constructs and returns a popup window.
	 * @param caption Popup window caption
	 */
	KPassivePopup *popup(const QString &caption = QString::null);

	/**
	 * Displays a popup message.
	 * @param reason Reason; icon type
	 * @param caption Popup title (bold)
	 * @param text Text to display
	 * @param timeout Timeout in seconds. Default value is 10 seconds.
	 * @param test @c TRUE if called from the settings dialog
	 */
	void popupMessage(const Reason reason, const QString &caption, const QString &text, const int timeout = 10, const bool test = FALSE);

	/**
	 * This is an overloaded member function.
	 * Uses main window caption as a popup title.
	 * @param reason Reason; icon type
	 * @param text Text to display
	 */
	void popupMessage(const Reason reason, const QString &text);

	/** Returns @c TRUE if the popup messages are enabled. */
	inline bool popupMessages() const { return _popupMessages; }

	/**
	 * Switches icon between idle and active states.
	 * @param yes If @c TRUE the state is "idle" (normal icon); otherwise the state is "active" (highlighted icon)
	 */	
	void setIdle(const bool yes);
	
	/** If @p yes is @c TRUE, then the popup messages are enabled. */
	inline void setPopupMessages(const bool yes) { _popupMessages = yes; }

protected:

	/** Disables/enables menu items before menu show. */
	virtual void contextMenuAboutToShow(KPopupMenu *menu);

private:
	bool
		_idle,
		_popupMessages;
	int
		cancelItem,
		lockScreenItem,
		logoutItem,
		rebootItem,
		shutDownItem,
		wizardItem,
		flashCount;
	KPassivePopup
		*_popup;
	KPixmap
		*activeIcon,
		*flashIcon;
	QLabel
		*popupCaption;
	QListBox
		*popupMessagesList;
	QPixmap
		*saveIcon;
	QTimer
		*flashTimer;
private slots:
	void flashTimeoutSlot();
	void lockScreenSlot();
	void logoutSlot();
	void rebootSlot();
	void shutDownSlot();
	void wizardSlot();
};

#endif // __MSYSTEMTRAY_H__
